# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import playerkit_pb2 as proto_dot_playerkit__pb2
from . import response_pb2 as proto_dot_response__pb2

GRPC_GENERATED_VERSION = '1.71.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in proto/playerkit_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class PlayerKitServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetAllOnlinePlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetAllOnlinePlayers',
                request_serializer=proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerByName = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerByName',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerByNameRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerByUUID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerByUUID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.ReleaseBindPlayer = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/ReleaseBindPlayer',
                request_serializer=proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerName = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerName',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerNameRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerEntityUniqueID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityUniqueID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt64Response.FromString,
                _registered_method=True)
        self.GetPlayerLoginTime = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerLoginTime',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt64Response.FromString,
                _registered_method=True)
        self.GetPlayerPlatformChatID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerPlatformChatID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerBuildPlatform = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerBuildPlatform',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt32Response.FromString,
                _registered_method=True)
        self.GetPlayerSkinID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerSkinID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanBuild = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanBuild',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanBuild = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanBuild',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanDig = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDig',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanDigRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanDig = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDig',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanDigRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanDoorsAndSwitches = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDoorsAndSwitches',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanDoorsAndSwitches = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDoorsAndSwitches',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanOpenContainers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOpenContainers',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanOpenContainers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOpenContainers',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanAttackPlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackPlayers',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanAttackPlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackPlayers',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanAttackMobs = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackMobs',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanAttackMobs = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackMobs',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanOperatorCommands = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOperatorCommands',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanOperatorCommands = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOperatorCommands',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanTeleport = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanTeleport',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanTeleport = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanTeleport',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusInvulnerable = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusInvulnerable',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusFlying = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusFlying',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusMayFly = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusMayFly',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerDeviceID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerDeviceID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerEntityRuntimeID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityRuntimeID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralUint64Response.FromString,
                _registered_method=True)
        self.GetPlayerEntityMetadata = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityMetadata',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerIsOP = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerIsOP',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerIsOPRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerOnline = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerOnline',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerOnlineRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SendPlayerChat = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerChat',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerChatRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerRawChat = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerRawChat',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerRawChatRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerTitle = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerTitle',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerTitleRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerActionBar = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerActionBar',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerActionBarRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.InterceptPlayerJustNextInput = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/InterceptPlayerJustNextInput',
                request_serializer=proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)


class PlayerKitServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetAllOnlinePlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerByName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerByUUID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReleaseBindPlayer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityUniqueID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerLoginTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerPlatformChatID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerBuildPlatform(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerSkinID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanBuild(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanBuild(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanDig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanDig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanDoorsAndSwitches(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanDoorsAndSwitches(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanOpenContainers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanOpenContainers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanAttackPlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanAttackPlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanAttackMobs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanAttackMobs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanOperatorCommands(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanOperatorCommands(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanTeleport(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanTeleport(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusInvulnerable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusFlying(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusMayFly(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerDeviceID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityRuntimeID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerIsOP(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerOnline(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerChat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerRawChat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerTitle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerActionBar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InterceptPlayerJustNextInput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PlayerKitServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetAllOnlinePlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllOnlinePlayers,
                    request_deserializer=proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerByName,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerByNameRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerByUUID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerByUUID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'ReleaseBindPlayer': grpc.unary_unary_rpc_method_handler(
                    servicer.ReleaseBindPlayer,
                    request_deserializer=proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerName,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerNameRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerEntityUniqueID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityUniqueID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt64Response.SerializeToString,
            ),
            'GetPlayerLoginTime': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerLoginTime,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt64Response.SerializeToString,
            ),
            'GetPlayerPlatformChatID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerPlatformChatID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerBuildPlatform': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerBuildPlatform,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt32Response.SerializeToString,
            ),
            'GetPlayerSkinID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerSkinID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanBuild,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanBuild,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanDig': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanDig,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanDigRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanDig': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanDig,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanDigRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanDoorsAndSwitches': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanDoorsAndSwitches,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanDoorsAndSwitches': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanDoorsAndSwitches,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanOpenContainers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanOpenContainers,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanOpenContainers': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanOpenContainers,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanAttackPlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanAttackPlayers,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanAttackPlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanAttackPlayers,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanAttackMobs': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanAttackMobs,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanAttackMobs': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanAttackMobs,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanOperatorCommands': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanOperatorCommands,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanOperatorCommands': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanOperatorCommands,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanTeleport': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanTeleport,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanTeleport': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanTeleport,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusInvulnerable': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusInvulnerable,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusFlying': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusFlying,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusMayFly': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusMayFly,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerDeviceID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerDeviceID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerEntityRuntimeID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityRuntimeID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralUint64Response.SerializeToString,
            ),
            'GetPlayerEntityMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityMetadata,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerIsOP': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerIsOP,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerIsOPRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerOnline': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerOnline,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerOnlineRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SendPlayerChat': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerChat,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerChatRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerRawChat': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerRawChat,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerRawChatRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerTitle': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerTitle,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerTitleRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerActionBar': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerActionBar,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerActionBarRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'InterceptPlayerJustNextInput': grpc.unary_unary_rpc_method_handler(
                    servicer.InterceptPlayerJustNextInput,
                    request_deserializer=proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fateark.proto.playerkit.PlayerKitService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('fateark.proto.playerkit.PlayerKitService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class PlayerKitService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetAllOnlinePlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetAllOnlinePlayers',
            proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerByName',
            proto_dot_playerkit__pb2.GetPlayerByNameRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerByUUID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerByUUID',
            proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReleaseBindPlayer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/ReleaseBindPlayer',
            proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerName',
            proto_dot_playerkit__pb2.GetPlayerNameRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityUniqueID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityUniqueID',
            proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerLoginTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerLoginTime',
            proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerPlatformChatID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerPlatformChatID',
            proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerBuildPlatform(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerBuildPlatform',
            proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt32Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerSkinID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerSkinID',
            proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanBuild',
            proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanBuild',
            proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanDig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDig',
            proto_dot_playerkit__pb2.GetPlayerCanDigRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanDig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDig',
            proto_dot_playerkit__pb2.SetPlayerCanDigRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanDoorsAndSwitches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDoorsAndSwitches',
            proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanDoorsAndSwitches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDoorsAndSwitches',
            proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanOpenContainers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOpenContainers',
            proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanOpenContainers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOpenContainers',
            proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanAttackPlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackPlayers',
            proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanAttackPlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackPlayers',
            proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanAttackMobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackMobs',
            proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanAttackMobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackMobs',
            proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanOperatorCommands(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOperatorCommands',
            proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanOperatorCommands(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOperatorCommands',
            proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanTeleport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanTeleport',
            proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanTeleport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanTeleport',
            proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusInvulnerable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusInvulnerable',
            proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusFlying(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusFlying',
            proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusMayFly(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusMayFly',
            proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerDeviceID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerDeviceID',
            proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityRuntimeID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityRuntimeID',
            proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralUint64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityMetadata',
            proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerIsOP(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerIsOP',
            proto_dot_playerkit__pb2.GetPlayerIsOPRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerOnline(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerOnline',
            proto_dot_playerkit__pb2.GetPlayerOnlineRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerChat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerChat',
            proto_dot_playerkit__pb2.SendPlayerChatRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerRawChat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerRawChat',
            proto_dot_playerkit__pb2.SendPlayerRawChatRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerTitle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerTitle',
            proto_dot_playerkit__pb2.SendPlayerTitleRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerActionBar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerActionBar',
            proto_dot_playerkit__pb2.SendPlayerActionBarRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InterceptPlayerJustNextInput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/InterceptPlayerJustNextInput',
            proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import os
import platform
import shlex
import subprocess
import threading
import time

from ...mc_bytes_packet.pool import bytes_packet_by_name
from ... import utils
from ...constants import SysStatus, PacketIDS
from ...packets import Packet_CommandOutput
from ...mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ...utils import fmts, sys_args, urlmethod
from ..types import UnreadyPlayer, Abilities
from .standard_launcher import StandardFrame
from .neo_libs import file_download as neo_fd, neo_conn
from .neo_libs.blob_hash.blob_hash_holder import (
    BlobHashHolder,
)


class FrameNeOmgAccessPoint(StandardFrame):
    """使用 NeOmega接入点 框架连接到游戏"""

    launch_type = "NeOmegaAccessPoint"

    def __init__(self) -> None:
        """初始化 NeOmega 框架

        Args:
            serverNumber (int): 服务器号
            password (str): 服务器密码
            fbToken (str): 验证服务器 Token
            auth_server (str): 验证服务器地址
        """
        super().__init__()
        self.status = SysStatus.LOADING
        self.neomg_proc = None
        self.serverNumber = None
        self.neomega_account_opt = None
        self.bot_name = ""
        self.omega = neo_conn.ThreadOmega(
            connect_type=neo_conn.ConnectType.Remote,
            address="tcp://localhost:24013",
            accountOption=None,
        )
        self.blob_hash_holder = BlobHashHolder(self.omega)
        self.serverNumber: int | None = None
        self.serverPassword: str | None = None
        self.fbToken: str | None = None
        self.auth_server: str | None = None
        self.exit_reason = ""

    def init(self):
        if "no-download-libs" not in sys_args.sys_args_to_dict().keys():
            fmts.print_inf("检测接入点和依赖库的最新版本..", end="\r")
            try:
                neo_fd.download_libs()
            except Exception as err:
                raise SystemExit(f"ToolDelta 因下载库异常而退出: {err}") from err
            fmts.print_inf("检测接入点和依赖库的最新版本..完成")
        else:
            fmts.print_war("将不会自动检测接入点依赖库的最新版本")
        neo_conn.load_lib()
        self.status = SysStatus.LAUNCHING

    def set_launch_data(
        self, serverNumber: int, password: str, fbToken: str, auth_server_url: str
    ):
        self.serverNumber = serverNumber
        self.serverPassword = password
        self.fbToken = fbToken
        self.auth_server = auth_server_url
        if self.serverNumber is None:
            self.neomega_account_opt = None
        else:
            self.neomega_account_opt = neo_conn.AccountOptions(
                AuthServer=self.auth_server,
                UserToken=self.fbToken,
                ServerCode=str(self.serverNumber),
                ServerPassword=self.serverPassword,
            )

    def set_omega_conn(self, addr: str) -> str:
        """设置 Omega 连接

        Args:
            openat_port (int): 端口号

        Raises:
            bool: 是否启动成功
        """
        retries = 1
        self.omega.address = addr
        MAX_RETRIES = 5  # 最大重试次数

        while retries <= MAX_RETRIES:
            try:
                self.omega.connect()
                retries = 1
                return ""
            except Exception as err:
                if self.status != SysStatus.LAUNCHING:
                    self.update_status(SysStatus.CRASHED_EXIT)
                    return "接入点无法连接"
                if "api not exist" in str(err):
                    fmts.print_inf("等待接入点连接中..", end="\r")
                    time.sleep(5)
                else:
                    fmts.print_war(f"OMEGA 连接失败: {err} (第{retries}次)")
                    time.sleep(5)
                    retries += 1
        fmts.print_err("最大重试次数已超过")
        self.update_status(SysStatus.CRASHED_EXIT)
        return "连接超时"

    def start_neomega_proc(self) -> int:
        """启动 NeOmega 进程

        Returns:
            int: 端口号
        """
        free_port = urlmethod.get_free_port(24013)
        sys_machine = platform.uname().machine
        if sys_machine == "x86_64":
            sys_machine = "amd64"
        elif sys_machine == "aarch64":
            sys_machine = "arm64"
        access_point_file = (
            f"neomega_{platform.uname().system.lower()}_access_point_{sys_machine}"
        )
        if "TERMUX_VERSION" in os.environ:
            access_point_file = f"neomega_android_access_point_{sys_machine}"
        if platform.system() == "Windows":
            access_point_file += ".exe"
        exe_file_path = os.path.join(os.getcwd(), "tooldelta", "bin", access_point_file)
        if platform.uname().system.lower() == "linux":
            os.system(f"chmod +x {shlex.quote(exe_file_path)}")
        # 只需要+x 即可
        if (
            self.serverNumber is None
            or self.serverPassword is None
            or self.fbToken is None
            or self.auth_server is None
        ):
            raise ValueError("未设置服务器号、密码、Token 或验证服务器地址")
        fmts.print_suc(f"将使用空闲端口 §f{free_port}§a 与接入点进行网络通信")
        self.neomg_proc = subprocess.Popen(
            [
                exe_file_path,
                "-server",
                str(self.serverNumber),
                "-T",
                self.fbToken,
                "-access-point-addr",
                f"tcp://localhost:{free_port}",
                "-server-password",
                str(self.serverPassword),
                "-auth-server",
                self.auth_server,
            ],
            encoding="utf-8",
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,
        )
        return free_port

    @utils.thread_func("NeOmega 信息显示线程", thread_level=utils.ToolDeltaThread.SYSTEM)
    def _msg_show_thread(self, launch_event: threading.Event) -> None:
        """显示来自 NeOmega 的信息"""
        if self.neomg_proc is None or self.neomg_proc.stdout is None:
            raise ValueError("接入点进程未启动")
        while True:
            msg_orig = self.neomg_proc.stdout.readline().strip("\n")
            if "成功完成网易要求的零知识机器人身份证明" in msg_orig:
                launch_event.set()
            if msg_orig in ("", "SIGNAL: exit"):
                fmts.print_with_info("接入点进程已结束", "§b NOMG ")
                if self.status == SysStatus.LAUNCHING:
                    self.update_status(SysStatus.CRASHED_EXIT)
                break
            fmts.print_with_info(msg_orig, "§b NOMG ")

    def launch(self) -> SystemExit | Exception | SystemError:
        """启动 NeOmega 进程

        returns:
            SystemExit: 正常退出
            Exception: 异常退出
            SystemError: 未知的退出状态
        """
        self.status = SysStatus.LAUNCHING
        self.exit_event = threading.Event()
        self.omega = neo_conn.ThreadOmega(
            connect_type=neo_conn.ConnectType.Remote,
            address="tcp://localhost:24013",
            accountOption=self.neomega_account_opt,
        )
        self.blob_hash_holder = BlobHashHolder(self.omega)
        openat_port = self.start_neomega_proc()
        launch_event = threading.Event()
        self._msg_show_thread(launch_event)
        if self.status != SysStatus.LAUNCHING:
            return SystemError("接入点无法连接到服务器")
        fmts.print_inf("等待接入点就绪..")
        while not launch_event.wait(timeout = 1):
            if self.exit_event.is_set():
                return SystemError("NeOmage 启动出现问题.")
            pass
        fmts.print_suc("接入点已就绪")
        if (err_str := self.set_omega_conn(f"tcp://127.0.0.1:{openat_port}")) == "":
            self.update_status(SysStatus.RUNNING)
            self.start_wait_omega_disconn_thread()
            fmts.print_suc("接入点框架通信网络连接成功")
            pcks = [
                self.omega.get_packet_id_to_name_mapping(i)
                for i in self.need_listen_packets
            ]
            self.omega.listen_packets(pcks, self.packet_handler_parent)
            self._exec_launched_listen_cbs()
        else:
            return SystemError(err_str)
        self.update_status(SysStatus.RUNNING)
        self.wait_crashed()
        if self.status == SysStatus.NORMAL_EXIT:
            return SystemExit("正常退出")
        if self.status == SysStatus.CRASHED_EXIT:
            return Exception("接入点进程已崩溃")
        return SystemError("未知的退出状态")

    def get_players_info(self):
        players_data: dict[str, UnreadyPlayer] = {}
        if self.status != SysStatus.RUNNING:
            raise ValueError("未连接到接入点")
        for i in self.omega.get_all_online_players():
            if i is not None:
                ab = Abilities(
                    i.can_build,
                    i.can_mine,
                    i.can_doors_and_switches,
                    i.can_open_containers,
                    i.can_attack_players,
                    i.can_attack_mobs,
                    i.can_operator_commands,
                    i.can_teleport,
                    0,  # TODO: player_permission 现在固定为 0
                    3 if i.op else 1,  # TODO: 除非玩家为 OP, 否则命令等级恒为 1
                )
                players_data[i.name] = UnreadyPlayer(
                    i.uuid,
                    i.entity_unique_id,
                    i.name,
                    i.uuid[-8:],
                    i.platform_chat_id,
                    i.device_id,
                    i.build_platform,
                    abilities=ab,
                )
            else:
                raise ValueError("未能获取玩家名和 UUID")
        return players_data

    def get_bot_name(self) -> str:
        """获取机器人名字

        Returns:
            str: 机器人名字
        """
        self.check_avaliable()
        if not self.bot_name:
            self.bot_name = self.omega.get_bot_basic_info().BotName
        return self.bot_name

    def packet_handler_parent(self, pkt_type: str, pkt: dict | bytes) -> None:
        """数据包处理器

        Args:
            pkt_type (str): 数据包类型
            pkt (dict): 数据包内容

        Raises:
            ValueError: 未连接到接入点
        """
        if (
            self.omega is None
            or self.dict_packet_handler is None
            or self.bytes_packet_handler is None
        ):
            raise ValueError("未连接到接入点")

        if type(pkt) is dict:
            packetType: int = self.omega.get_packet_name_to_id_mapping(pkt_type)  # type: ignore
            self.dict_packet_handler(packetType, pkt)
        elif type(pkt) is bytes:
            real_pkt = bytes_packet_by_name(pkt_type)
            real_pkt.decode(pkt)
            self.bytes_packet_handler(real_pkt.real_packet_id(), real_pkt)

    def check_avaliable(self):
        if self.status != SysStatus.RUNNING:
            raise ValueError("未连接到游戏")

    def sendcmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以玩家身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Returns:
            Optional[Packet_CommandOutput]: 返回命令结果
        """
        self.check_avaliable()
        if waitForResp:
            res = self.omega.send_player_command_need_response(cmd, timeout)
            if res is None:
                raise TimeoutError("指令超时")
            return res
        self.omega.send_player_command_omit_response(cmd)
        return None

    def sendwscmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以玩家身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Returns:
            Optional[Packet_CommandOutput]: 返回命令结果
        """
        self.check_avaliable()
        if waitForResp:
            res = self.omega.send_websocket_command_need_response(cmd, timeout)
            if res is None:
                raise TimeoutError(f"指令超时: {cmd}")
            return res
        self.omega.send_websocket_command_omit_response(cmd)
        return None

    def sendwocmd(self, cmd: str) -> None:
        """以 wo 身份发送命令

        Args:
            cmd (str): 命令

        Raises:
            NotImplementedError: 未实现此方法
        """
        self.check_avaliable()
        self.omega.send_settings_command(cmd)

    def sendPacket(self, pckID: int, pck: dict | BaseBytesPacket) -> None:
        """发送数据包

        Args:
            pckID (int): 数据包 ID
            pck (dict | BaseBytesPacket): 数据包内容dict
        """
        self.check_avaliable()
        if type(pck) is dict:
            self.omega.send_game_packet_in_json_as_is(pckID, pck)
        else:
            self.omega.send_game_packet_in_bytes(pckID, pck.encode())  # type: ignore

    def blobHashHolder(self) -> BlobHashHolder:
        """blobHashHolder 返回当前结点的 Blob hash cache 缓存数据集的持有人

        Returns:
            BlobHashHolder: 当前结点的 Blob hash cache 缓存数据集的持有人
        """
        return self.blob_hash_holder

    def place_command_block_with_nbt_data(
        self,
        block_name: str,
        block_states: str,
        position: tuple[int, int, int],
        nbt_data: neo_conn.CommandBlockNBTData,
    ):
        """在 position 放置方块名为 block_name 且方块状态为 block_states 的命令块，
        同时向该方块写入 nbt_data 所指代的 NBT 数据

        Args:
            block_name (str): 命令块的方块名，如 chain_command_block
            block_states (str): 命令块的方块状态，如 朝向南方 的命令方块表示为 ["facing_direction":3]
            position (tuple[int, int, int]): 命令块应当被放置的位置。三元整数元组从左到右依次对应世界坐标的 X, Y, Z 轴坐标
            nbt_data (neo_conn.CommandBlockNBTData): 该命令块的原始 NBT 数据

        Raises:
            NotImplementedError: 未实现此方法
        """
        self.check_avaliable()
        self.omega.place_command_block(
            neo_conn.CommandBlockPlaceOption(
                X=position[0],
                Y=position[1],
                Z=position[2],
                BlockName=block_name,
                BockState=block_states,
                NeedRedStone=(not nbt_data.ConditionalMode),
                Conditional=nbt_data.ConditionalMode,
                Command=nbt_data.Command,
                Name=nbt_data.CustomName,
                TickDelay=nbt_data.TickDelay,
                ShouldTrackOutput=nbt_data.TrackOutput,
                ExecuteOnFirstTick=nbt_data.ExecuteOnFirstTick,
            )
        )

    @utils.thread_func("检测 Omega 断开连接线程", utils.ToolDeltaThread.SYSTEM)
    def start_wait_omega_disconn_thread(self):
        self.exit_reason = self.omega.wait_disconnect()
        if self.status == SysStatus.RUNNING:
            self.update_status(SysStatus.CRASHED_EXIT)

    def reload_listen_packets(self, listen_packets: set[PacketIDS]) -> None:
        super().reload_listen_packets(listen_packets)
        pcks = [
            self.omega.get_packet_id_to_name_mapping(i)
            for i in self.need_listen_packets
        ]
        self.omega.listen_packets(pcks, self.packet_handler_parent)

    sendPacketJson = sendPacket



"插件市场客户端"

import asyncio
import os
import time
import traceback
import requests
import json

from .utils import cfg, fmts
from .constants import (
    TOOLDELTA_CLASSIC_PLUGIN,
    TOOLDELTA_PLUGIN_CFG_DIR,
    TOOLDELTA_PLUGIN_DATA_DIR,
    PLUGIN_MARKET_SOURCE_OFFICIAL,
)
from .plugin_load import PluginRegData, PluginsPackage
from .utils import try_int, thread_gather, urlmethod


FILETREE = dict[str, "int | FILETREE"]

def url_join(*urls: str) -> str:
    return "/".join(url.strip("/") for url in urls).strip("/")


def unfold_directory_dict(dirs: FILETREE, base_path: str = "", sep: str = "/"):
    unfolded: list[str] = []
    for dirname, dir_or_file in dirs.items():
        dirpath = sep.join((base_path, dirname)).strip("/")
        if isinstance(dir_or_file, dict):
            unfolded.extend(unfold_directory_dict(dir_or_file, dirpath, sep))
        else:
            unfolded.append(dirpath)
    return unfolded

def get_json_from_url(url: str):
    try:
        resp = requests.get(url, timeout=5)
        resp.raise_for_status()
        return resp.json()
    except requests.RequestException as exc:
        raise requests.RequestException(
            f"URL 请求失败: {url} \n§6(看起来您要更改配置文件中的链接)\n报错: {exc}"
        ) from exc
    except json.JSONDecodeError as exc:
        raise requests.RequestException(
            f"服务器返回了不正确的答复：{resp.text}"
        ) from exc


class PluginMarket:
    "插件市场类"

    def __init__(self):
        self._cached_market_tree = {}
        self._cached_plugins_id_map = {}
        self._cached_market_filetree: FILETREE = {}
        try:
            self.plugin_market_content_url = cfg.get_cfg(
                "ToolDelta基本配置.json", {"插件市场源": str}
            )["插件市场源"]
        except Exception:
            self.plugin_market_content_url = PLUGIN_MARKET_SOURCE_OFFICIAL

    def enter_plugin_market(self, source_url: str | None = None, in_game=False) -> None:
        """
        进入插件市场

        Args:
            source_url (str | None, optional): 插件市场源
            in_game (bool, optional): 是否在游戏内调用的插件市场命令
        """
        if source_url:
            self.plugin_market_content_url = source_url

        fmts.ansi_save_screen()
        fmts.clean_print("§6正在连接到插件市场..")
        CONTENT_LENGTH = 15

        try:
            market_datas = self.get_market_tree()
            plugin_ids_map = self.get_plugin_id_name_map()
            show_list = [
                (i, j) if i.startswith("[pkg]") else ("[pkg]" + i, j)
                for i, j in market_datas["Packages"].items()
            ] + list(market_datas["MarketPlugins"].items())

            while 1:
                fmts.ansi_cls()
                valid_show_list = self.search_by_rule(market_datas, show_list)
                if valid_show_list is None:
                    fmts.clean_print("§6已退出。")
                    return
                elif valid_show_list == []:
                    input(fmts.clean_fmt("§c未找到匹配的插件; 回车键继续"))
                    continue

                total_indexes = len(valid_show_list)
                now_index = 0
                sum_pages = (total_indexes - 1) // CONTENT_LENGTH + 1
                last_operation = ""

                while 1:
                    self.display_plugins_and_packages(
                        market_datas,
                        plugin_ids_map,
                        valid_show_list,
                        now_index,
                        sum_pages,
                        CONTENT_LENGTH,
                    )

                    last_operation = (
                        input(
                            fmts.clean_fmt("§f回车键继续上次操作, §bq§f 退出，请输入: ")
                        )
                        or last_operation
                    )
                    last_operation = last_operation.lower().strip()
                    if last_operation in ["+", "-"]:
                        now_index = max(
                            0,
                            min(
                                now_index
                                + (
                                    CONTENT_LENGTH
                                    if last_operation == "+"
                                    else -CONTENT_LENGTH
                                ),
                                total_indexes - 1,
                            ),
                        )
                    elif last_operation == "q":
                        break
                    else:
                        res = try_int(last_operation)
                        if res and 1 <= res <= total_indexes:
                            result = valid_show_list[res - 1]
                            if not result[0].startswith("[pkg]"):
                                # 这是插件
                                plugin_data = self.get_plugin_data_from_market(
                                    result[0]
                                )
                                if self.handle_plugin_selection(plugin_data):
                                    break
                            else:
                                # 这是整合包
                                package_data = self.get_package_data_from_market(
                                    result[0]
                                )
                                if self.handle_package_selection(package_data):
                                    break
                        else:
                            fmts.clean_print("§c超出序号范围")

        except (KeyError, requests.RequestException) as err:
            fmts.clean_print(
                f"§c获取插件市场插件出现问题({err.__class__.__name__}): {err}"
            )
            input(fmts.clean_fmt("§6按回车键继续.."))
        except Exception:
            fmts.clean_print("§c获取插件市场插件出现问题, 报错如下:")
            fmts.clean_print("§c" + traceback.format_exc().replace("\n", "\n§c"))
            input(fmts.clean_fmt("§6按回车键继续.."))
        finally:
            fmts.ansi_load_screen()
            fmts.clean_print("§a已从插件市场返回 ToolDelta 控制台。")

    @staticmethod
    def search_by_rule(
        market_datas, show_list: list[tuple[str, dict]]
    ) -> list[tuple[str, dict]] | None:
        fmts.clean_print(f"{market_datas['SourceName']}: {market_datas['Greetings']}")
        fmts.clean_print("§a------------------------------")
        fmts.clean_print("§6请选择搜索方式: ")
        fmts.clean_print("  1 -     §b按插件名")
        fmts.clean_print("  2 -     §d按插件作者名")
        fmts.clean_print("  3 -     §e按插件 ID")
        fmts.clean_print("  4 -     §a随便逛逛")
        fmts.clean_print("  其它    §c退出")
        resp = input(fmts.clean_fmt("请输入选项: ")).strip().strip("[]")
        output_show_list: list[tuple[str, dict]] = []
        match resp:
            case "1":
                plugin_name_kw = (
                    input(fmts.clean_fmt("§6请输入插件名(中的关键词): "))
                    .strip()
                    .lower()
                )
                if plugin_name_kw == "":
                    return []
                for plugin_id, plugin_data in show_list:
                    pname = (
                        plugin_id
                        if plugin_id.startswith("[pkg]")
                        else plugin_data["name"]
                    )
                    if plugin_name_kw in pname.lower():
                        output_show_list.append((plugin_id, plugin_data))
                return output_show_list
            case "2":
                plugin_author_kw = (
                    input(fmts.clean_fmt("§6请输入插件作者名(中的关键词): "))
                    .strip()
                    .lower()
                )
                if plugin_author_kw == "":
                    return []
                for plugin_name, plugin_data in show_list:
                    if plugin_author_kw in plugin_data["author"].lower():
                        output_show_list.append((plugin_name, plugin_data))
                return output_show_list
            case "3":
                plugin_id_kw = (
                    input(fmts.clean_fmt("§6请输入插件ID(中的关键词): "))
                    .strip()
                    .lower()
                )
                if plugin_id_kw == "":
                    return []
                for plugin_id, plugin_data in show_list:
                    if plugin_id_kw in plugin_id:
                        output_show_list.append((plugin_id, plugin_data))
                        break
                return output_show_list
            case "4":
                return show_list
            case _:
                return None

    def display_plugins_and_packages(
        self,
        market_datas: dict,
        plugin_ids_map: dict[str, str],
        show_list: list[tuple[str, dict]],
        start_index: int,
        total_pages: int,
        content_length: int = 15,
    ):
        """
        显示插件列表

        Args:
            start_index (int): 起始索引
            total_pages (int): 总页数
        """
        fmts.ansi_cls()
        fmts.clean_print(f"{market_datas['SourceName']}: {market_datas['Greetings']}")
        for i in range(start_index, min(start_index + content_length, len(show_list))):
            show_name, description = show_list[i]
            if show_name.startswith("[pkg]"):
                pkg_name = show_name
                fmts.clean_print(f" {i + 1}. §c[整合包]§e{pkg_name[5:]}")
            else:
                plugin_id = show_name
                plugin_name = plugin_ids_map[plugin_id]
                plugin_type = {"classic": "类式"}.get(
                    description.get("plugin-type", "unknown"),
                    description.get("plugin-type", "unknown"),
                )
                fmts.clean_print(
                    f" {i + 1}. §e{plugin_name} §av{description['version']} "
                    f"§b@{description['author']} §d{plugin_type}插件"
                )
        fmts.clean_print(
            f"§f第§a{start_index // content_length + 1}§f/§a{total_pages}§f页, 输入§b+§f/§b-§f翻页"
        )
        fmts.clean_print("§f输入插件序号选中插件并查看其下载页")

    def handle_package_selection(self, pack: PluginsPackage):
        ok = self.skim_package(pack)
        if ok:
            fmts.clean_print("可以输入 §breload§r 使这个整合包生效哦")
            return (
                input(fmts.clean_fmt("§f输入 §cq §f退出, 其他则返回插件市场")).lower()
                == "q"
            )
        else:
            fmts.clean_print("已取消。")
            time.sleep(1)
        return False

    def handle_plugin_selection(self, plugin_data: PluginRegData):
        ok, pres = self.skim_plugin(plugin_data)
        if ok:
            fmts.clean_print("可以输入 §breload§r 使这个插件生效哦")
            return (
                input(fmts.clean_fmt("输入 §cq §r退出, 其他则返回插件市场")).lower()
                == "q"
            )
        else:
            fmts.clean_print("已取消。")
            time.sleep(1)
        return False

    # 从插件市场的 market_tree.json 获取数据
    def get_market_tree(self) -> dict:
        if self._cached_market_tree != {}:
            return self._cached_market_tree
        market_datas = self._cached_market_tree = get_json_from_url(
            url_join(self.plugin_market_content_url, "market_tree.json")
        )
        return market_datas

    # 从插件市场获取单个插件数据
    def get_plugin_data_from_market(self, plugin_id: str) -> PluginRegData:
        plugin_name = self.get_plugin_id_name_map().get(plugin_id)
        if plugin_name is None:
            raise requests.RequestException(
                f"无法通过 ID: {plugin_id} 查找插件, 你可能需要反馈此问题至开发者"
            )
        data_url = url_join(self.plugin_market_content_url, plugin_name, "datas.json")
        datas = get_json_from_url(data_url)
        return PluginRegData(plugin_name, datas)

    # 从插件市场获取单个整合包数据
    def get_package_data_from_market(self, name: str) -> PluginsPackage:
        target_data_url = url_join(self.plugin_market_content_url, name, "datas.json")
        content = requests.get(target_data_url).json()
        return PluginsPackage(name, content)

    def skim_plugin(
        self, plugin_data: PluginRegData
    ) -> tuple[bool, list[PluginRegData]]:
        """
        选中插件进行介绍与操作

        Args:
            plugin_data (PluginRegData): 插件注册数据

        Returns:
            tuple[bool, list[PluginRegData]]: 是否下载，下载的插件列表
        """
        pre_plugins_str = (
            ", ".join([f"{k}§7v{v}" for k, v in plugin_data.pre_plugins.items()])
            or "无"
        )
        has_doc = self.get_plugin_filetree(plugin_data.name).get("readme.txt") is not None
        while True:
            fmts.ansi_cls()
            fmts.clean_print(f"{plugin_data.name} v{plugin_data.version_str}")
            fmts.clean_print(
                f"作者: §f{plugin_data.author}§7, 版本: §f{plugin_data.version_str} §b{plugin_data.plugin_type_str}"
            )
            fmts.clean_print(f"前置插件：§f{pre_plugins_str}")
            fmts.clean_print(f"介绍：{plugin_data.description}")
            fmts.clean_print("")
            prompt = f"§f下载 = §aY§f, 取消 = §cN§f{', 查看文档 = §dD§f ' if has_doc else ''} 请输入: "
            res = (
                input(fmts.clean_fmt(prompt))
                .lower()
                .strip()
            )
            if res == "y":
                fmts.clean_print(f"§6正在下载插件：§f{plugin_data.name}", end="\r")
                pres = self.download_plugin(plugin_data)
                pres.reverse()
                return True, pres
            elif res == "d":
                if has_doc:
                    fmts.clean_print("§6正在读取文档..")
                    self.lookup_plugin_doc(plugin_data)
                else:
                    fmts.clean_print("§c该插件没有文档..")
                return False, []
            else:
                return False, []

    def skim_package(self, pack: PluginsPackage) -> bool:
        """
        选中整合包进行介绍与操作

        Args:
            pack (PluginsPackage): 整合包数据类

        Returns:
            bool: 是否下载安装
        """
        fmts.ansi_cls()
        inc_plugins_name: list[str] = []
        for pid in pack.plugin_ids:
            if pname := self.get_plugin_id_name_map().get(pid):
                inc_plugins_name.append(pname)
            else:
                fmts.clean_print(f"§c无法通过ID {pid} 查找插件, 有可能是插件市场出错")
                return True
        fmts.clean_print(f"§f整合包 §b{pack.name[5:]} §7(v{pack.version})§r:")
        fmts.clean_print(f"作者: §b{pack.author}")
        fmts.clean_print("介绍: §f" + pack.description.replace("\n", "\n      "))
        fmts.clean_print("§d包含的插件的列表:")
        for pname in inc_plugins_name:
            fmts.clean_print(f" §7- §r{pname}")
        # 显示其他文件数量
        ftree = self.get_market_filetree()
        dirdata = ftree.get(pack.name)
        if dirdata is None:
            raise ValueError(f"插件市场内不存在整合包 {pack.name}")
        plugin_config_files = ftree.get(url_join(pack.name, "插件配置文件"), {})
        assert not isinstance(plugin_config_files, int)
        # 计算插件配置文件数量
        config_files_num = len(unfold_directory_dict(plugin_config_files))
        # 计算插件数据文件数量
        data_file_dir = ftree.get(url_join(pack.name, "插件数据文件"), {})
        assert not isinstance(data_file_dir, int)
        data_files_num = len(unfold_directory_dict(data_file_dir))
        fmts.clean_print(
            f"§2并包含§r{config_files_num}§2个插件配置文件, §r{data_files_num}§2个插件数据文件"
        )
        if (
            input(fmts.clean_fmt("§f下载安装 = §aY§f, 取消 = §cN§f, 请输入："))
            .lower()
            .strip()
        ) == "y":
            self.download_plugin_package(pack)
            return True
        else:
            return False

    def download_plugin_package(self, pack: PluginsPackage):
        fmts.clean_print("§6获取插件数据中...", end="\r")
        find_plugins = thread_gather(
            [(self.get_plugin_data_from_market, (i,)) for i in pack.plugin_ids]
        )
        ftree = self.get_market_filetree()
        dirdata = ftree.get(pack.name)
        if dirdata is None:
            raise ValueError(f"插件市场内不存在整合包 {pack.name}")
        plugin_config_files = ftree.get(url_join(pack.name, "插件配置文件"), {})
        plugin_data_files = ftree.get(url_join(pack.name, "插件数据文件"), {})
        assert not isinstance(plugin_config_files, int)
        assert not isinstance(plugin_data_files, int)
        download_url_dirs: list[tuple[str, str]] = []
        # 插件配置文件
        for cfgfile_path in unfold_directory_dict(plugin_config_files):
            f_url = url_join(
                self.plugin_market_content_url,
                pack.name,
                "插件配置文件",
                cfgfile_path,
            )
            f_local = os.path.join(TOOLDELTA_PLUGIN_CFG_DIR, cfgfile_path)
            if os.path.isfile(f_local) and (
                input(
                    fmts.clean_fmt(
                        f"§6配置文件 §r{cfgfile_path}§6 已存在, 是否替换§r(§a[默认]y§r/§cn§r)§6: "
                    )
                )
                .strip()
                .lower()
                != "n"
            ):
                download_url_dirs.append((f_url, f_local))
        # 插件数据文件
        for inc_file_path in unfold_directory_dict(plugin_data_files):
            # url: [pkg]pkg_name/插件数据文件/anydir/...
            # local: 插件数据文件/anydir/...
            f_url = url_join(
                self.plugin_market_content_url, pack.name, "插件数据文件", inc_file_path
            )
            f_local = os.path.join(
                TOOLDELTA_PLUGIN_DATA_DIR, inc_file_path
            )
            if not os.path.isfile(f_local) or (
                input(
                    fmts.clean_fmt(
                        f"§6数据文件 §r{f_local}§6 已存在, 是否替换§r(§ay§r/§cn[默认]§r)§6: "
                    )
                )
                .strip()
                .lower()
                == "y"
            ):
                download_url_dirs.append((f_url, f_local))
        fmts.clean_print(f"§6开始下载整合包 {pack.name.replace('[pkg]', '')}")
        for _, fpath in download_url_dirs:
            # 初始化需要的文件夹路径
            os.makedirs(os.path.dirname(fpath), exist_ok=True)
        asyncio.run(urlmethod.download_file_urls(download_url_dirs))
        # 下载插件主体
        for plugin in find_plugins:
            self.download_plugin(plugin)
        fmts.clean_print("整合包下载完成")

    # 下载插件
    def download_plugin(
        self, plugin_data: PluginRegData, with_pres=True, is_enabled=False
    ) -> list[PluginRegData]:
        fmts.clean_print(
            f"§6正在获取 §f{plugin_data.name} §6插件的下载任务清单.." + " " * 15,
            end="\r",
        )
        if with_pres:
            plugin_list = self.get_plugin_download_list(plugin_data)
        else:
            plugin_list = {plugin_data.name: plugin_data}
        plugin_filepaths_dict: dict[str, list[str]] = {}
        for plugin_name, plugin_data in plugin_list.items():
            plugin_filepaths_dict[plugin_name] = unfold_directory_dict(
                self.get_plugin_filetree(plugin_data.name)
            )
        fmts.clean_print(f"§a已获取插件下载清单 §f{plugin_data.name}§a" + " " * 15)
        plugin_remote_to_local_path: list[tuple[str, str]] = []
        for plugin_name, this_plugin_info in plugin_list.items():
            match this_plugin_info.plugin_type:
                case "classic":
                    plugintype_path = os.path.join("插件文件", TOOLDELTA_CLASSIC_PLUGIN)
                case _:
                    raise ValueError(
                        f"未知插件类型：{this_plugin_info.plugin_type}, 你可能需要通知 ToolDelta 项目开发组解决"
                    )
            for filepath in plugin_filepaths_dict[plugin_name]:
                plugin_remote_to_local_path.append(
                    (
                        url_join(self.plugin_market_content_url, this_plugin_info.name, filepath),
                        os.path.join(plugintype_path, this_plugin_info.name, filepath),
                    )
                )
        fmts.clean_print(
            f"§bTD下载管理器: §7需要下载 §c{len(plugin_remote_to_local_path)} §7个文件"
        )
        asyncio.run(urlmethod.download_file_urls(plugin_remote_to_local_path))
        fmts.clean_print("§a• 插件安装已完成")
        return list(plugin_list.values())

    def lookup_plugin_doc(self, plugin: PluginRegData):
        url = url_join(self.plugin_market_content_url, plugin.name, "readme.txt")
        resp = requests.get(url)
        if resp.status_code != 200:
            fmts.clean_print("§c无法获取插件文档")
            return
        counter = 0
        fmts.ansi_cls()
        fmts.clean_print("§b文档正文:")
        for ln in resp.text.split("\n"):
            fmts.clean_print("  " + ln)
            counter += 1
            MAX_LINES = 15
            if counter > MAX_LINES:
                counter = 0
                input(fmts.clean_fmt("§a[按回车键继续阅读..]"))
        input(fmts.clean_fmt("§a已经读完正文了 [Enter]"))

    # 获取插件 ID 到插件名的映射
    def get_plugin_id_name_map(self) -> dict[str, str]:
        if self._cached_plugins_id_map == {}:
            try:
                res = get_json_from_url(
                    url_join(self.plugin_market_content_url, "plugin_ids_map.json")
                )
            except Exception as err:
                fmts.clean_print(
                    f"§c从 {self.plugin_market_content_url} 获取插件信息遇到问题: {err}"
                )
                raise SystemExit
            self._cached_plugins_id_map = res
            return res
        else:
            return self._cached_plugins_id_map

    # 获取一个插件所需下载的所有插件 -> dict[插件名, 插件数据]
    def get_plugin_download_list(
        self, plugin_data: PluginRegData
    ) -> dict[str, PluginRegData]:
        download_paths: dict[str, PluginRegData] = {}
        stack = [plugin_data]
        while stack:
            current_plugin = stack.pop()
            download_paths[current_plugin.name] = current_plugin
            for plugin_id in current_plugin.pre_plugins:
                plugin_datas = self.get_plugin_data_from_market(plugin_id)
                stack.append(plugin_datas)
        return download_paths

    # 获取插件市场的文件目录结构
    def get_market_filetree(self) -> FILETREE:
        if self._cached_market_filetree == {}:
            self._cached_market_filetree = get_json_from_url(
                url_join(self.plugin_market_content_url, "directory_tree.json")
            )
        return self._cached_market_filetree

    # 获取单个插件的插件文件夹目录结构
    def get_plugin_filetree(self, plugin_name: str) -> FILETREE:
        tree = self.get_market_filetree().get(plugin_name)
        if tree is None:
            raise KeyError(f"无法通过名称: {plugin_name} 获取此插件下载项")
        if isinstance(tree, dict):
            return tree
        else:
            raise ValueError(f"名称 {plugin_name} 是文件而非目录")

    # 根据插件 ID 获取插件的最新版本号
    def get_latest_plugin_version(self, plugin_id: str) -> tuple[int, int, int]:
        result = get_json_from_url(
            url_join(self.plugin_market_content_url, "latest_versions.json")
        ).get(plugin_id)
        if result is None:
            raise KeyError(f"无法通过 ID: {plugin_id} 获取最新插件版本")
        try:
            ver = tuple(int(i) for i in result.split("."))
            assert len(ver) == 3
        except Exception:
            raise ValueError(f"插件版本号字符串 {result!r} 不正确")
        return ver[0], ver[1], ver[2]


market = PluginMarket()



from typing import Callable
from tooldelta.internal.launch_cli.neo_libs.blob_hash.define import BaseBlobHashHolder
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.define import (
    HashWithPosition,
    PayloadByHash,
)


class MirrorWorldHandler:
    """
    MirrorWorldHandler 指示镜像存档资源持有人所使用的处理函数，
    它们被用于处理来自客户端的 blob cache 查询和同步请求
    """

    base_blob_hash_holder: BaseBlobHashHolder
    f1: Callable[[list[HashWithPosition]], list[bool]] | None
    f2: Callable[[list[HashWithPosition]], list[PayloadByHash]] | None
    f3: Callable[[list[PayloadByHash]], None] | None
    f4: Callable[[list[HashWithPosition]], None] | None
    f5: Callable[[], None] | None

    def __init__(self, base_blob_hash_holder: BaseBlobHashHolder):
        """
        Args:
            base_blob_hash_holder (BaseBlobHashHolder): Blob hash holder 的底层实现
        """
        self.base_blob_hash_holder = base_blob_hash_holder
        self.f1 = None
        self.f2 = None
        self.f3 = None
        self.f4 = None
        self.f5 = None

    def set_handler(
        self,
        handle_query_disk_hash_exist: (
            Callable[[list[HashWithPosition]], list[bool]] | None
        ),
        handle_get_disk_hash_payload: (
            Callable[[list[HashWithPosition]], list[PayloadByHash]] | None
        ),
        handle_require_sync_hash_to_disk: Callable[[list[PayloadByHash]], None] | None,
        handle_clean_blob_hash_and_apply_to_world: (
            Callable[[list[HashWithPosition]], None] | None
        ),
        handle_server_disconnect: Callable[[], None] | None,
    ):
        """set_handler 根据给定的函数设置处理器，然后镜像存档的持有人便可作为资源中心处理来自服务者的资源请求

        Args:
            handle_query_disk_hash_exist (Callable[[list[HashWithPosition]], list[bool]] | None):
                handle_query_disk_hash_exist 处理来自服务者的 QueryDiskHashExist 请求。
                仅被镜像存档的持有人所使用。

                QueryDiskHashExist 用于向镜像存档持有人发起检索请求，
                目的仅在于检索 hashes 是否命中镜像存档中的存储。

                需要关注的是，handle_query_disk_hash_exist 的实现者需要确保底层实现确实是仅查询，
                这意味着 handle_query_disk_hash_exist 的具体实现不应该包含加载具体子区块的成分，
                而只是单纯的检查 blob hash 值是否可以与镜像存档中记录的值匹配

            handle_get_disk_hash_payload (Callable[[list[HashWithPosition]], list[PayloadByHash]] | None):
                handle_get_disk_hash_payload 处理来自服务者的 GetDiskHashPayload 请求
                仅被镜像存档的持有人所使用。

                GetDiskHashPayload 用于从镜像存档持有人请求 hashes 的二进制数据荷载。

                handle_get_disk_hash_payload 的实现者应当确保此函数返回的结果都满足 xxhash.Sum64(payload) = hash。
                如果镜像存档的持有人发现记录的 blob hash 值与传入 handle_get_disk_hash_payload 的 hashes 匹配，
                但相应的 payload 算出的 hash 值却不是给定的 blob hash 值时，说明这个子区块可能已经被更改。
                这种情况相当的罕见，例如发生在用户打开镜像存档并对这些子区块作出实际修改之后

            handle_require_sync_hash_to_disk (Callable[[list[PayloadByHash]], None] | None):
                handle_require_sync_hash_to_disk 处理来自服务者的 RequireSyncHashToDisk 请求
                仅被镜像存档的持有人所使用。

                RequireSyncHashToDisk 用于命令镜像存档持有人将 payload 指示的 (hash, payload) 储存到镜像存档中。

                可以保证 RequireSyncHashToDisk 只由服务者提出，并且服务者将尽可能少的提出命令，以减少造成的磁盘读写开销。
                这意味着，服务者通过一系列方法确定出了那些真正需要被更改的子区块，然后向镜像存档的持有人提出同步更改的命令

            handle_clean_blob_hash_and_apply_to_world (Callable[[list[HashWithPosition]], None] | None):
                handle_clean_blob_hash_and_apply_to_world 由镜像存档持有人的底层直接调用，
                底层将不断监听 packet.SubChunk，然后从中找到全部都是空气的子区块，
                并作为参数传入到 handle_clean_blob_hash_and_apply_to_world 之中。

                由于全为空气的子区块的 blob hash 是不存在的，所以传入的 pos 切片中，
                对于其中的每个元素 x 而言，x.Hash 是未被使用的字段（或者，x.Hash 总是 0）。
                基于此，我们需要这样一个函数来初始化那些全为空气的子区块
                （即，handleGetDiskHashPayload 只会同步非空子区块，而这个函数的功能则是补齐同步空气子区块的部分）。

                除了初始化以外，本函数还有另外一个目的，便是把原本在镜像存档中不非空的子区块
                变为非空，这可能发生在服务器内某个子区块从非空变为空的过程中。

                需要提醒的是，每个区块中完全是空气的子区块可能非常多，因为 Minecraft 中大多数地方都是空气。
                于是，这个函数在很大可能上被大量的调用并传入大量的应该被设置为空气的子区块条目。
                这意味着，如果机器人不断的走过相同的区块，然后就很可能得到相同的完全是空的子区块，
                然后重复地传入到 handle_clean_blob_hash_and_apply_to_world 之中。

                handle_clean_blob_hash_and_apply_to_world 的实现者需要确保相应的实现不会对磁盘造成大量读写，
                这意味着在真正设置一个子区块为空子区块前，应当通过一些手段确定这个子区块是否是空的。
                如果不是空的（或者这个子区块在镜像存档中不存在），再进行读写；反之，不进行任何操作。

                可以将空子区块的 blob hash 简单置为 0，因此对于非空子区块，它们的 blob hash 一定非 0。
                这意味着，如果一个子区块被要求设为空气，而这个子区块不存在 blob hash 或 blob hash 非 0，
                那么这一定这个子区块在镜像存档中的情况一定满足下面其中一个
                    - 这个子区块不存在于镜像存档
                    - 这个子区块在镜像存档中非空
                于是，可以安全的把这个镜像存档的这个子区块的 blob hash 置为 0，并将这个子区块置空

            handle_server_disconnect (Callable[[], None] | None):
                当 blob hash 缓存数据集的服务者撤销当前镜像存档持有人的持有身份时，
                底层实现将会尝试进行恢复（尝试重新取得我们作为持有人的身份）。

                当恢复工作失败后，handle_server_disconnect 将会被调用，
                以确保作为曾是持有人的自己可以进行一部分收尾工作。

                handle_server_disconnect 一般不会发生，除非服务者认为我们作为持有者已经死亡
        """
        self.f1 = handle_query_disk_hash_exist
        self.f2 = handle_get_disk_hash_payload
        self.f3 = handle_require_sync_hash_to_disk
        self.f4 = handle_clean_blob_hash_and_apply_to_world
        self.f5 = handle_server_disconnect



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .. import ToolDelta


class ExtendFunction:
    def __init__(self, frame: "ToolDelta"):
        self.frame = frame

    def when_activate(self): ...



from tooldelta.constants.packets import PacketIDS
from tooldelta.mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from tooldelta.mc_bytes_packet.level_chunk import LevelChunk
from tooldelta.mc_bytes_packet.sub_chunk import SubChunk
from tooldelta.mc_bytes_packet.sub_chunk_request import SubChunkRequest


def is_bytes_packet(real_packet_id: int) -> bool:
    return (
        real_packet_id == PacketIDS.IDSubChunkRequest
        or real_packet_id == PacketIDS.IDLevelChunk
        or real_packet_id == PacketIDS.IDSubChunk
    )


def bytes_packet_by_name(custom_packet_name: str) -> BaseBytesPacket:
    if custom_packet_name == "SubChunkRequest":
        return SubChunkRequest()
    elif custom_packet_name == "LevelChunk":
        return LevelChunk()
    return SubChunk()



from collections.abc import Callable
from typing import TYPE_CHECKING
from threading import Event

from ...utils import fmts, create_result_cb
from ..types.player import Player, UnreadyPlayer
from ..types.player_abilities import Abilities, update_player_ability_from_ability_data
from ...constants import PacketIDS

if TYPE_CHECKING:
    from tooldelta import ToolDelta
    from ..packet_handler import PacketHandler

KeyUUID = str
KeyUniqueID = int


class PlayerInfoMaintainer:
    """
    用于维护和更新玩家信息。
    """

    def __init__(self, frame: "ToolDelta"):
        self.frame = frame
        self.name_to_player: dict[str, Player] = {}
        self.uq_to_player: dict[int, Player] = {}
        self.uuid_to_player: dict[str, Player] = {}
        self.xuid_to_player: dict[str, Player] = {}
        self.player_abilities: dict[KeyUniqueID, Abilities] = {}
        self.player_abilities_getter_callback: dict[
            KeyUniqueID, Callable[[bool], None]
        ] = {}
        self.inited_event = Event()
        # pendings
        self.pending_player_abilities_packet: dict[KeyUniqueID, dict] = {}
        self.pending_add_player_packet: dict[KeyUUID, dict] = {}

    def hook_packet_handler(self, packet_handler: "PacketHandler"):
        # PlayerList 的优先级是最高的, 至少需要比 PluginGroup 高
        # 以在插件事件执行完成之前先行完善玩家信息
        packet_handler.add_dict_packet_listener(
            packet_id=PacketIDS.PlayerList, cb=self._hook_playerlist, priority=100
        )
        packet_handler.add_dict_packet_listener(
            packet_id=PacketIDS.UpdateAbilities,
            cb=self._hook_update_abilities,
            priority=100,
        )
        packet_handler.add_dict_packet_listener(
            packet_id=PacketIDS.AddPlayer,
            cb=self._hook_add_player,
            priority=100,
        )

    def block_init(self):
        # 载入初始玩家数据
        if not self.name_to_player:
            playerinfs = self.frame.launcher.get_players_info()
            if playerinfs is not None:
                for unready_player in playerinfs.values():
                    self.add_player(unready_player)
                fmts.print_suc("已从接入点获取全局玩家数据")
            else:
                self._block_until_inited()

    def getBotInfo(self) -> Player:
        """
        获取机器人本身的玩家对象。

        Raises:
            ValueError: 无法通过机器人名称获取信息

        Returns:
            Player: 玩家对象
        """
        bot_inf = self.name_to_player.get(self.frame.game_ctrl.bot_name)
        if bot_inf is None:
            raise ValueError("无法获取机器人信息")
        return bot_inf

    def getPlayerByName(self, name: str) -> Player | None:
        """
        通过玩家名称获取玩家对象。

        Args:
            name (str): 玩家名

        Returns:
            Player | None: 玩家对象
        """
        return self.name_to_player.get(name)

    def getPlayerByUniqueID(self, uqID: int) -> Player | None:
        """
        通过玩家uniqueID获取玩家对象。

        Args:
            uqID (int): uniqueID

        Returns:
            Player | None: 玩家对象
        """
        return self.uq_to_player.get(uqID)

    def getPlayerByUUID(self, uuid: str) -> Player | None:
        """
        通过玩家UUID获取玩家对象。
        Args:
            uuid (str): UUID

        Returns:
            Player | None: 玩家对象
        """
        return self.uuid_to_player.get(uuid)

    def getPlayerByXUID(self, uuid: str) -> Player | None:
        """
        通过玩家XUID获取玩家对象。
        Args:
            uuid (str): XUID

        Returns:
            Player | None: 玩家对象
        """
        return self.xuid_to_player.get(uuid)

    def getAllPlayers(self) -> list[Player]:
        """
        获取所有当前在线的玩家的玩家对象列表。

        Returns:
            list[Player]: 玩家对象列表
        """
        return list(self.name_to_player.values())

    def add_player(self, player: UnreadyPlayer):
        ready_player = player.ready(self)
        self.name_to_player[player.name] = ready_player
        self.uq_to_player[player.unique_id] = ready_player
        self.uuid_to_player[player.uuid] = ready_player
        self.xuid_to_player[player.xuid] = ready_player
        self._lookup_pendings(ready_player)

    def remove_player(self, player: Player):
        del self.name_to_player[player.name]
        del self.uq_to_player[player.unique_id]
        del self.uuid_to_player[player.uuid]
        del self.xuid_to_player[player.xuid]
        if player.unique_id in self.player_abilities:
            del self.player_abilities[player.unique_id]
        player.online = False

    def get_player_ability(self, player: Player) -> Abilities:
        ab = self.player_abilities.get(player.unique_id)
        if ab is None:
            getter, setter = create_result_cb()
            self.player_abilities_getter_callback[player.unique_id] = setter
            res = getter(20)
            del self.player_abilities_getter_callback[player.unique_id]
            if res is None:
                raise ValueError("[internal] 获取玩家能力超时")
            return self.player_abilities[player.unique_id]
        else:
            return ab

    def _block_until_inited(self):
        # 阻塞直到获取到包含全局玩家身份的 PlayerList 数据包
        fmts.print_inf("正在等待获取全局玩家数据..")
        ok = self.inited_event.wait(30)
        if ok:
            fmts.print_suc("已收到全局玩家数据")
        else:
            raise SystemExit("全局玩家数据获取超时")

    def _hook_update_abilities(self, packet: dict):
        uqID = packet["AbilityData"]["EntityUniqueID"]
        player = self.getPlayerByUniqueID(uqID)
        if player is None:
            self.pending_player_abilities_packet[uqID] = packet
        else:
            self._update_player_by_ability_packet(player, packet)
        return False

    def _hook_add_player(self, packet: dict):
        puuid = packet["UUID"]
        p = self.getPlayerByUUID(puuid)
        if p is None:
            self.pending_add_player_packet[puuid] = packet
        else:
            self._update_player_by_add_player_packet(p, packet)
        return False

    def _hook_playerlist(self, packet: dict):
        self.inited_event.set()
        if packet["ActionType"] == 0:
            for entry in packet["Entries"]:
                self._hook_playerlist_add_player(entry)
        else:
            for entry in packet["Entries"]:
                self._hook_playerlist_remove_player(entry)
        return False

    def _hook_playerlist_add_player(self, entry: dict):
        unique_id = entry["EntityUniqueID"]
        playername = entry["Username"]
        self.add_player(
            UnreadyPlayer(
                uuid=entry["UUID"],
                unique_id=unique_id,
                name=playername,
                xuid=entry["XUID"],
                platform_chat_id=entry["PlatformChatID"],
                device_id=None,
                build_platform=entry["BuildPlatform"],
                online=True,
            )
        )

    def _hook_playerlist_remove_player(self, entry: dict):
        if player := self.getPlayerByUUID(entry["UUID"]):
            self.remove_player(player)
        else:
            fmts.print_war(
                f"[internal] PlayerInfoMaintainer: remove_player: 找不到玩家的 UUID: {entry['UUID']}"
            )

    def _lookup_pendings(self, player: Player):
        if player.uuid in self.pending_add_player_packet:
            self._update_player_by_add_player_packet(
                player, self.pending_add_player_packet.pop(player.uuid)
            )
        if player.unique_id in self.pending_player_abilities_packet:
            self._update_player_by_ability_packet(
                player, self.pending_player_abilities_packet.pop(player.unique_id)
            )

    def _update_player_by_ability_packet(self, player: Player, packet: dict):
        ab_data = packet["AbilityData"]
        uqID = ab_data["EntityUniqueID"]
        update_player_ability_from_ability_data(self, player, ab_data)
        if uqID in self.player_abilities_getter_callback:
            self.player_abilities_getter_callback[uqID](True)

    def _update_player_by_add_player_packet(self, player: Player, packet: dict):
        player.runtime_id = packet["EntityRuntimeID"]
        player.device_id = packet["DeviceID"]
        update_player_ability_from_ability_data(self, player, packet["AbilityData"])

    def __iter__(self):
        "iter online players"
        return iter(self.getAllPlayers())



from ...constants import SysStatus, tooldelta_cfg
from ...utils import cfg, fmts, sys_args
from .neomega_access_point import FrameNeOmgAccessPoint


class FrameNeOmgAccessPointRemote(FrameNeOmgAccessPoint):
    """远程启动器框架 (使用 NeOmega接入点 框架的 Remote 连接)"""

    launch_type = "NeOmegaAccessPoint Remote"

    def launch(self) -> SystemExit | Exception | SystemError:
        """启动远程启动器框架

        Raises:
            AssertionError: 端口号错误

        Returns:
            SystemExit | Exception | SystemError: 退出状态
        """
        cfgs = cfg.get_cfg("ToolDelta基本配置.json", tooldelta_cfg.LAUNCH_CFG_STD)
        openat_addr = sys_args.sys_args_to_dict().get("access-point-port") or cfgs.get(
            "NeOmega远程接入点模式", {}
        ).get("远程连接地址", "tcp://127.0.0.1:24020")
        fmts.print_inf(f"将从[{openat_addr}]连接至游戏网络接入点, 等待接入中...")
        if (err_str := self.set_omega_conn(openat_addr)) == "":
            self.update_status(SysStatus.RUNNING)
            self.start_wait_omega_disconn_thread()
            fmts.print_suc("已与接入点进程建立通信网络")
            pcks = []
            for i in self.need_listen_packets:
                try:
                    pcks.append(self.omega.get_packet_id_to_name_mapping(i))
                except KeyError:
                    fmts.print_war(f"无法监听数据包: {i}")
            self.omega.listen_packets(pcks, self.packet_handler_parent)
            self._exec_launched_listen_cbs()
            fmts.print_suc("ToolDelta 待命中")
        else:
            return SystemError(err_str)
        self.wait_crashed()
        if self.status == SysStatus.NORMAL_EXIT:
            return SystemExit("正常退出。")
        if self.status == SysStatus.CRASHED_EXIT:
            return Exception(f"接入点已崩溃: {self.exit_reason}")
        return SystemError("未知的退出状态")



import struct
import numpy

from dataclasses import dataclass, field
from io import BytesIO


@dataclass(frozen=True)
class SubChunkPos:
    x: int = 0
    y: int = 0
    z: int = 0


@dataclass(frozen=True)
class HashWithPosY:
    hash: int = 0
    pos_y: int = 0


@dataclass(frozen=True)
class HashWithPosition:
    hash: int = 0
    sub_chunk_pos: SubChunkPos = field(default_factory=lambda: SubChunkPos())
    dimension: int = 0

    def encode(self, writer: BytesIO):
        writer.write(
            struct.pack(
                "<Qiii",
                self.hash,
                self.sub_chunk_pos.x,
                self.sub_chunk_pos.y,
                self.sub_chunk_pos.z,
            )
        )
        writer.write(self.dimension.to_bytes(length=1, byteorder="little"))


def decode_hash_with_position(reader: BytesIO) -> HashWithPosition:
    hash, x, y, z = struct.unpack("<Qiii", reader.read(20))
    dimension = reader.read(1)[0]
    return HashWithPosition(hash, SubChunkPos(x, y, z), dimension)


@dataclass
class PayloadByHash:
    hash: HashWithPosition = field(default_factory=lambda: HashWithPosition())
    payload: numpy.ndarray = field(
        default_factory=lambda: numpy.array([], dtype=numpy.uint8)
    )

    def encode(self, writer: BytesIO):
        self.hash.encode(writer)
        writer.write(struct.pack("<I", len(self.payload)))
        writer.write(self.payload.tobytes())

    def decode(self, reader: BytesIO):
        self.hash = decode_hash_with_position(reader)
        self.payload = numpy.frombuffer(
            reader.read(struct.unpack("<I", reader.read(4))[0]), dtype=numpy.uint8
        )



from . import tempjson
from .fbtoken import fbtokenFix, if_token
from .locks import ChatbarLock, players_in_chatbar_lock
from .packet_transition import get_playername_and_msg_from_text_packet
from .safe_json import (
    DataReadError,
    safe_json_dump,
    safe_json_load,
    read_from_plugin,
    write_to_plugin,
)
from .system_safe_close import safe_close
from .timer_events import timer_event, timer_event_boostrap
from .tooldelta_thread import ToolDeltaThread, createThread, thread_func, thread_gather
from .basic import (
    simple_fmt,
    simple_assert,
    try_int,
    try_convert,
    fuzzy_match,
    split_list,
    fill_list_index,
    remove_mc_color_code,
    to_plain_name,
    to_player_selector,
    create_result_cb,
    create_desperate_attr_class,
)

# ---------------- 向下兼容 ---------------


def loadPathJson(
    path: str,
    needFileExists: bool = True,
    timeout: int = 60,
    default=None,
):
    tempjson.load_from_path(
        path, need_file_exists=needFileExists, unload_delay=timeout, default=default
    )


def read_as_tmp(
    path: str,
    needFileExists: bool = True,
    timeout: int = 60,
    default=None,
):
    tempjson.load_and_read(
        path, need_file_exists=needFileExists, timeout=timeout, default=default
    )


def write_as_tmp(
    path: str,
    obj,
    needFileExists: bool = True,
    timeout: int = 60,
    default=None,
):
    tempjson.load_and_write(path, obj, need_file_exists=needFileExists, timeout=timeout)


TMPJson = create_desperate_attr_class(
    "TMPJson",
    [
        loadPathJson,
        read_as_tmp,
        write_as_tmp,
        tempjson.read,
        tempjson.write,
        tempjson.flush,
        tempjson.cancel_change,
        tempjson.get,
        tempjson.get_tmps,
    ],
)
r"""
:class:`TMPJson` 已弃用。
如果想使用其中的函数，请改为使用 `from tooldelta.utils import tempjson`。
"""

TMPJson.unloadPathJson = tempjson.unload_to_path

JsonIO = create_desperate_attr_class("JsonIO", [])
r"""
:class:`JsonIO` 已弃用。
如果想使用其中的函数，请改为使用 `from tooldelta.utils import safe_json`。
"""

JsonIO.readFileFrom = read_from_plugin
JsonIO.writeFileTo = write_to_plugin
JsonIO.SafeJsonLoad = safe_json_load
JsonIO.SafeJsonDump = safe_json_dump
JsonIO.DataReadError = DataReadError

Utils = create_desperate_attr_class(
    "Utils",
    [
        JsonIO,
        TMPJson,
        ChatbarLock,
        ToolDeltaThread,
        simple_fmt,
        simple_assert,
        try_int,
        try_convert,
        fuzzy_match,
        split_list,
        fill_list_index,
        remove_mc_color_code,
        to_plain_name,
        to_player_selector,
        create_result_cb,
        thread_func,
        thread_gather,
        timer_event,
    ],
)
r"""
:class:`Utils` 已弃用。
如果想使用其中的函数，请改为使用 `from tooldelta import utils`。
"""

Utils.createThread = createThread
chatbar_lock_list = players_in_chatbar_lock

# ruff: noqa: F401



"ToolDelta 常量定义"

from enum import IntEnum, Enum

class TextType(IntEnum):
    TextTypeRaw = 0
    TextTypeChat = 1
    TextTypeTranslation = 2
    TextTypePopup = 3
    TextTypeJukeboxPopup = 4
    TextTypeTip = 5
    TextTypeSystem = 6
    TextTypeWhisper = 7
    TextTypeAnnouncement = 8
    TextTypeObjectWhisper = 9
    TextTypeObject = 10
    TextTypeObjectAnnouncement = 11


class ContainerType(IntEnum):
    ContainerTypeInventory = -1  # 物品栏
    ContainerTypeContainer = 0  # 容器
    ContainerTypeWorkbench = 1  # 工作台
    ContainerTypeFurnace = 2  # 熔炉
    ContainerTypeEnchantment = 3  # 附魔台
    ContainerTypeBrewingStand = 4  # 酿造台
    ContainerTypeAnvil = 5  # 铁砧
    ContainerTypeDispenser = 6  # 发射器
    ContainerTypeDropper = 7  # 投掷器
    ContainerTypeHopper = 8  # 漏斗
    ContainerTypeCauldron = 9  # 炼药锅
    ContainerTypeCartChest = 10  # 箱子矿车
    ContainerTypeCartHopper = 11  # 漏斗矿车
    ContainerTypeHorse = 12  # 马
    ContainerTypeBeacon = 13  # 信标
    ContainerTypeStructureEditor = 14  # 结构编辑器
    ContainerTypeTrade = 15  # 交易
    ContainerTypeCommandBlock = 16  # 命令方块
    ContainerTypeJukebox = 17  # 唱片机
    ContainerTypeArmour = 18  # 盔甲
    ContainerTypeHand = 19  # 手持
    ContainerTypeCompoundCreator = 20  # 化合物创建器
    ContainerTypeElementConstructor = 21  # 元素构造器
    ContainerTypeMaterialReducer = 22  # 材料分解器
    ContainerTypeLabTable = 23  # 实验台
    ContainerTypeLoom = 24  # 织布机
    ContainerTypeLectern = 25  # 讲台
    ContainerTypeGrindstone = 26  # 磨石
    ContainerTypeBlastFurnace = 27  # 高炉
    ContainerTypeSmoker = 28  # 烟熏炉
    ContainerTypeStonecutter = 29  # 切石机
    ContainerTypeCartography = 30  # 制图台
    ContainerTypeHUD = 31  # HUD
    ContainerTypeJigsawEditor = 32  # 拼图编辑器
    ContainerTypeSmithingTable = 33  # 锻造台
    ContainerTypeChestBoat = 34  # 箱子船
    ContainerTypeDecoratedPot = 35  # 装饰罐
    ContainerTypeCrafter = 36  # 工匠


class PlayerActionType(IntEnum):
    PlayerActionStartBreak = 0
    PlayerActionAbortBreak = 1
    PlayerActionStopBreak = 2
    PlayerActionGetUpdatedBlock = 3
    PlayerActionDropItem = 4
    PlayerActionStartSleeping = 5
    PlayerActionStopSleeping = 6
    PlayerActionRespawn = 7
    PlayerActionJump = 8
    PlayerActionStartSprint = 9
    PlayerActionStopSprint = 10
    PlayerActionStartSneak = 11
    PlayerActionStopSneak = 12
    PlayerActionCreativePlayerDestroyBlock = 13
    PlayerActionDimensionChangeDone = 14
    PlayerActionStartGlide = 15
    PlayerActionStopGlide = 16
    PlayerActionBuildDenied = 17
    PlayerActionCrackBreak = 18
    PlayerActionChangeSkin = 19
    PlayerActionSetEnchantmentSeed = 20
    PlayerActionStartSwimming = 21
    PlayerActionStopSwimming = 22
    PlayerActionStartSpinAttack = 23
    PlayerActionStopSpinAttack = 24
    PlayerActionStartBuildingBlock = 25
    PlayerActionPredictDestroyBlock = 26
    PlayerActionContinueDestroyBlock = 27
    PlayerActionStartItemUseOn = 28
    PlayerActionStopItemUseOn = 29
    PlayerActionHandledTeleport = 30
    PlayerActionMissedSwing = 31
    PlayerActionStartCrawling = 32
    PlayerActionStopCrawling = 33


class EffectIDS(str, Enum):
    EffectSpeed = "speed"  # 速度
    EffectSlowNess = "slowness"  # 缓慢
    EffectHaste = "haste"  # 急迫
    EffectMiningFatigue = "mining_fatigue"  # 挖掘疲劳
    EffectStrength = "strength"  # 力量
    EffectInstantHealth = "instant_health"  # 瞬间治疗
    EffectInstantDamage = "instant_damage"  # 瞬间伤害
    EffectJumpBoost = "jump_boost"  # 跳跃提升
    EffectNausea = "nausea"  # 反胃
    EffectRegenerAtion = "regeneration"  # 生命恢复
    EffectResistance = "resistance"  # 耐力
    EffectFireResistance = "fire_resistance"  # 火焰抗性
    EffectWaterBreathing = "water_breathing"  # 水下呼吸
    EffectInvisibility = "invisibility"  # 隐身
    EffectBlindness = "blindness"  # 致盲
    EffectNightVision = "night_vision"  # 夜视
    EffectHunger = "hunger"  # 饥饿
    EffectWeakness = "weakness"  # 虚弱
    EffectPoison = "poison"  # 中毒
    EffectWither = "wither"  # 凋零
    EffectHealthBoost = "health_boost"  # 生命提升
    EffectAbsorption = "absorption"  # 吸收
    EffectSaturation = "saturation"  # 饱和
    EffectLevitation = "levitation"  # 漂浮
    EffectFatalPoison = "fatal_poison"  # 中毒(致命)
    EffectConduitPower = "conduit_power"  # 潮涌能量
    EffectSlowFalling = "slow_falling"  # 缓慢落下
    EffectBadOmen = "bad_omen"  # 不详之兆
    EffectVillageHero = "village_hero"  # 村庄英雄
    EffectDarkNess = "darkness"  # 黑暗


class BuildStructureIDS(str, Enum):
    StructureBuriedTreasure = "buried_treasure"  # 埋藏宝藏
    StructureEndcity = "endcity"  # 末地城
    StructureFortress = "fortress"  # 下界要塞
    StructureMansion = "mansion"  # 林地府邸
    StructureMineShaft = "mineshaft"  # 废弃矿井
    StructureMonuMent = "monument"  # 海底遗迹
    StructurePillagerOutpost = "pillager_outpost"  # 掠夺者前哨站
    StructureRuins = "ruins"  # 水下遗迹
    StructureShipWreck = "shipwreck"  # 沉船
    StructureStrongHold = "stronghold"  # 要塞
    StructureTemple = "temple"  # 沙漠神殿/丛林神殿/沼泽小屋/雪屋
    StructureVillage = "village"  # 村庄



import json
import os
from typing import Any
from . import fmts

PathLike = str | os.PathLike[str]

def safe_write(filepath: PathLike, obj: Any, indent=2):
    """
    安全地写入文件, 防止文件在写入途中被强行中止。

    Args:
        filepath (TextIOWrapper): 文件路径
        obj (Any): 写入的 json 待序列化对象
    """
    while 1:
        try:
            bak_name = str(filepath) + ".bak"
            content = json.dumps(obj, indent=indent, ensure_ascii=False)
            with open(bak_name, "w", encoding="utf-8") as fp:
                fp.write(content)
                break
        except BaseException as err:
            # 防止写入过程中被退出, 导致文件只写了一半
            fmts.print_war(f"文件在写入时遭到强行中止 ({err}), 重试")
            continue
    try:
        os.remove(filepath)
    except FileNotFoundError:
        pass
    os.rename(bak_name, filepath)



r"""
# ToolDelta: NEMC Rental Server R-Plugin Loader

网易我的世界手机版 租赁服机器人式插件加载器

- 进入 ToolDelta 界面:

    ```python
    from tooldelta import client_title

    client_title()
    ```
- 快速启动 ToolDelta:

    ```python
    from tooldelta import start_tool_delta

    start_tool_delta()
    ```
- 快速创建插件所需库环境

    ```python
    from tooldelta import *
    ```
"""

__all__ = [
    "TYPE_CHECKING",
    "Builtins",
    "Config",
    "Frame",
    "GameCtrl",
    "Plugin",
    "Print",
    "ToolDelta",
    "ToolDelta",
    "client_title",
    "start_tool_delta",
    "tooldelta",
]

from typing import TYPE_CHECKING

from . import utils
from .frame import GameCtrl, ToolDelta
from .internal.types import Player, Chat, FrameExit, InternalBroadcast
from .launch_options import client_title
from .plugin_load.classic_plugin import Plugin, plugin_entry
from .starter import start_tool_delta, tooldelta
from .utils import cfg, fmts, Utils
from .version import get_tool_delta_version, check_tooldelta_version

# 重定向
Builtins = Utils
Config = cfg
Frame = ToolDelta
Print = fmts

__all__ = [
    "TYPE_CHECKING",
    "Builtins",
    "Chat",
    "Config",
    "Frame",
    "FrameExit",
    "GameCtrl",
    "InternalBroadcast",
    "Player",
    "Plugin",
    "Print",
    "ToolDelta",
    "ToolDelta",
    "check_tooldelta_version",
    "client_title",
    "fmts",
    "get_tool_delta_version",
    "plugin_entry",
    "start_tool_delta",
    "tooldelta",
    "utils"
]



import grpc
import json
import uuid
import importlib

from ....constants import PacketIDS
from ....internal.types import UnreadyPlayer, Abilities

utils_pb2 = importlib.import_module(".proto.utils_pb2", package=__package__)
reversaler_pb2 = importlib.import_module(".proto.reversaler_pb2", package=__package__)
listener_pb2 = importlib.import_module(".proto.listener_pb2", package=__package__)
playerkit_pb2 = importlib.import_module(".proto.playerkit_pb2", package=__package__)

from .proto.listener_pb2_grpc import ListenerServiceStub
from .proto.reversaler_pb2_grpc import FateReversalerServiceStub
from .proto.utils_pb2_grpc import UtilsServiceStub
from .proto.playerkit_pb2_grpc import PlayerKitServiceStub


grpc_con: grpc.Channel | None = None
utils_stub: UtilsServiceStub | None = None
listener_stub: ListenerServiceStub | None = None
playerkit_stub: PlayerKitServiceStub | None = None

listen_packets: set[int] = set()


def get_grpc_con():
    global grpc_con
    if grpc_con is None:
        raise Exception("在建立连接前调用")
    return grpc_con


def get_utils_stub():
    global utils_stub
    if utils_stub is None:
        raise Exception("在建立连接前调用")
    return utils_stub


def get_listener_stub():
    global listener_stub
    if listener_stub is None:
        raise Exception("在建立连接前调用")
    return listener_stub


def get_playerkit_stub():
    global playerkit_stub
    if playerkit_stub is None:
        raise Exception("在建立连接前调用")
    return playerkit_stub


def connect(address: str) -> None:
    global grpc_con, listener_stub, playerkit_stub, utils_stub
    grpc_con = grpc.insecure_channel(address)
    utils_stub = UtilsServiceStub(grpc_con)
    listener_stub = ListenerServiceStub(grpc_con)
    playerkit_stub = PlayerKitServiceStub(grpc_con)


def login(
    auth_server: str,
    fbtoken: str,
    server_code: str,
    server_password: str,
) -> tuple[int, str, str]:
    request = reversaler_pb2.NewFateReversalerRequest(
        auth_server=auth_server,
        user_name="",
        user_password="",
        user_token=fbtoken,
        server_code=server_code,
        server_password=server_password,
    )
    resp = FateReversalerServiceStub(get_grpc_con()).NewFateReversaler(request)
    return resp.status, resp.payload, resp.error_msg


def read_output():
    for response in get_listener_stub().ListenFateArk(
        listener_pb2.ListenFateArkRequest()
    ):
        yield response.msg_type, response.msg, response.err_msg


def read_packet():
    for packet in get_listener_stub().ListenPackets(
        listener_pb2.ListenPacketsRequest()
    ):
        pk_payload = packet.payload
        if pk_payload == "":
            continue
        yield packet.id, json.loads(packet.payload)


def sendPacket(pkID: int, pk: dict):
    get_utils_stub().SendPacket(
        utils_pb2.SendPacketRequest(
            packet_id=pkID, json_str=json.dumps(pk, ensure_ascii=False)
        )
    )


def set_listen_packets(pkIDs: set[int]):
    global listen_packets
    for i in pkIDs:
        if i not in listen_packets:
            resp = get_listener_stub().ListenTypedPacket(
                listener_pb2.ListenTypedPacketRequest(packet_id=i)
            )
            if resp.status != 0:
                raise Exception(resp.error_msg)
            listen_packets.add(i)


def sendcmd_and_get_uuid(cmd: str):
    ud = str(uuid.uuid4())
    sendPacket(
        PacketIDS.CommandRequest,
        {
            "CommandLine": cmd,
            "CommandOrigin": {
                "Origin": 0,
                "UUID": ud,
                "RequestID": ud,
                "PlayerUniqueID": 0,
            },
            "Internal": False,
            "Version": 0x24,
            "UnLimited": False,
        },
    )
    return ud


def sendwscmd_and_get_uuid(cmd: str):
    ud = str(uuid.uuid4())
    sendPacket(
        PacketIDS.CommandRequest,
        {
            "CommandLine": cmd,
            "CommandOrigin": {
                "Origin": 5,
                "UUID": ud,
                "RequestID": ud,
                "PlayerUniqueID": 0,
            },
            "Internal": False,
            "Version": 0x24,
            "UnLimited": False,
        },
    )
    return ud


def sendwocmd(cmd: str):
    sendPacket(
        PacketIDS.SettingsCommand,
        {"CommandLine": cmd, "SuppressOutput": False},
    )


def get_online_player_uuids() -> list[str]:
    return json.loads(
        get_playerkit_stub()
        .GetAllOnlinePlayers(playerkit_pb2.GetAllOnlinePlayersRequest())
        .payload
    )


def get_unready_player(uuid: str) -> UnreadyPlayer:
    stub = get_playerkit_stub()
    ab = Abilities(
        stub.GetPlayerCanBuild(
            playerkit_pb2.GetPlayerCanBuildRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanDig(
            playerkit_pb2.GetPlayerCanDigRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanDoorsAndSwitches(
            playerkit_pb2.GetPlayerCanDoorsAndSwitchesRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanOpenContainers(
            playerkit_pb2.GetPlayerCanOpenContainersRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanAttackPlayers(
            playerkit_pb2.GetPlayerCanAttackPlayersRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanAttackMobs(
            playerkit_pb2.GetPlayerCanAttackMobsRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanOperatorCommands(
            playerkit_pb2.GetPlayerCanOperatorCommandsRequest(uuid_str=uuid)
        ).payload,
        stub.GetPlayerCanTeleport(
            playerkit_pb2.GetPlayerCanTeleportRequest(uuid_str=uuid)
        ).payload,
        0,  # TODO: player_permission 现在固定为 0
        (
            3
            if stub.GetPlayerIsOP(
                playerkit_pb2.GetPlayerIsOPRequest(uuid_str=uuid)
            ).payload
            else 1
        ),  # TODO: 除非玩家为 OP, 否则命令等级恒为 1
    )
    return UnreadyPlayer(
        uuid=uuid,
        unique_id=stub.GetPlayerEntityUniqueID(
            playerkit_pb2.GetPlayerEntityUniqueIDRequest(uuid_str=uuid)
        ).payload,
        name=stub.GetPlayerName(
            playerkit_pb2.GetPlayerNameRequest(uuid_str=uuid)
        ).payload,
        xuid=uuid[-8:],
        platform_chat_id=stub.GetPlayerPlatformChatID(
            playerkit_pb2.GetPlayerPlatformChatIDRequest(uuid_str=uuid)
        ).payload,
        device_id=stub.GetPlayerDeviceID(
            playerkit_pb2.GetPlayerDeviceIDRequest(uuid_str=uuid)
        ).payload,
        build_platform=stub.GetPlayerBuildPlatform(
            playerkit_pb2.GetPlayerBuildPlatformRequest(uuid_str=uuid)
        ).payload,
        online=stub.GetPlayerOnline(
            playerkit_pb2.GetPlayerOnlineRequest(uuid_str=uuid)
        ).payload,
        abilities=ab,
    )


def get_bot_name():
    return json.loads(
        UtilsServiceStub(get_grpc_con())
        .GetClientMaintainedBotBasicInfo(
            utils_pb2.GetClientMaintainedBotBasicInfoRequest()
        )
        .payload
    )["BotName"]
